;;;  EDL_ZAHN.LSP, 05-11-1996, Ing. Edlinger, VACE-Wien
;;;
;;;  30-04-97	Layerstruktur neu (L_SET)
;;;
;;;  Start mit: ZAHN

; Original: zahn.lsp


; Diese Lisproutine zeichnet einen Evolventenzahnkranz auf dem Layer "ZR" als
; Polylinie, dessen Mittelpunkt dem Einfgepunkt entspricht. Der Teilkreis
; und das Zentrumkreuz werden auf dem Layer "MITTE" (sofern vorhanden) erzeugt.
; Es werden die Anzahl der Zhne, das Modul, der Eingriffwinkel, das Kopfspiel
; und die Zahnfurundung abgefragt.
; Die Vorgaben in den Abfragen knnen leicht eigenen Bedrfnissen angepasst
; werden. Hierzu sind die entsprechenden "cond"-Anweisungen (direkt nach dem
; jeweiligen Abfragebefehl) zu editieren. Beispiel: In der Abfrage zum Kopfspiel
; wird in der ihr folgenden Zeile "(cond ((null c) (setq c (* 0.25 m))))" die
; Zahl 0.25 durch den neuen Standardwert als Faktor fr das Modul m ersetzt.
; Desgleichen in der entsprechenden Abfrage selbst.
; Die Genauigkeit bei der Konstruktion der Evolvente fr die Zahnflanke bildet
; den Winkel (N-ten Teil von 15 Grad), mit dem die Eingrifflinie auf dem
; Grundkreis abgerollt wird .
;                                                       Wittenberg den 13.10.92
;                                                                   Denys Lipke

;------------------------------------------------------------------------------
(defun Fehler ()
 (textscr)
 (princ "\n\n Fehler - Funktion abgegrochen")
 (princ "\n Eingriffwinkel oder Furundung zu gro\n")
 (setvar "blipmode" sblip)                      ;Rcksetzen der Systemvariablen
 (setvar "cmdecho" scmde)
 (setvar "clayer" oldlayer)
 (quit)
 (princ)
)
;------------------------------------------------------------------------------
(defun C:ZAHN ()

 (initget 1)
 (setq mp (getpoint "\n Einfgepunkt : "))
 (initget (+ 1 2 4))
 (setq z (getint "\n Zhnezahl : "))
 (initget (+ 1 2 4))
 (setq m (getreal "\n Modul : "))
 (initget (+ 2 4))
 (setq egwin (getreal "\n Eingriffswinkel <20> : "))
 (cond ((null egwin) (setq egwin 20.0)))
 (initget 4)
 (setq c (getreal "\n Kopfspiel <0.25m> : "))
 (cond ((null c) (setq c (* 0.25 m))))
 (initget 4)
 (setq zfr (getreal "\n Zahnfurundung <0.38m> : "))
 (cond ((null zfr) (setq zfr (* 0.38 m))))
 (initget (+ 2 4))
 (setq genau (getreal "\n Genauigkeit der Evolvente fr die Zahnflanke <5>: "))
 (cond ((null genau) (setq genau 5.0)))
  (list
   (setq uel (getreal "\n berlnge des Zentrumkreuzes <5> : "))
   (cond ((null uel) (setq uel 5)))
  )
;------------------------------------------------------------------------------
 (setq scmde (getvar "cmdecho"))                       ;sichern Systemvariablen
 (setq sblip (getvar "blipmode"))
 (setq oldlayer (getvar "clayer"))                          ;aktuellen Layer sichern
 (setvar "cmdecho" 0)                                         ;ndern Variablen
 (setvar "blipmode" 0)
;------------------------------------------------------------------------------
                         ;Bestimmungsgren
 (setq dt (* z m))                                         ;Teikreisdurchmesser
 (setq rt (/ dt 2.0))
 (setq df (- dt (* 2.0 (+ m c))))                          ;Fukreisdurchmesser                 
 (setq rf (/ df 2.0))
 (setq dk (+ dt (* 2.0 m)))                               ;Kopfkreisdurchmesser
 (setq rk (/ dk 2.0))
 (setq dg (* (cos (/ (* egwin pi) 180.0)) dt))           ;Grundkreisdurchmesser
 (setq rg (/ dg 2.0))
 (setq p (* m pi))                                                     ;Teilung
 (if (< rg (+ rf zfr)) (fehler))                              ;Eingabekontrolle
;------------------------------------------------------------------------------
                         ;zeichnen Kopfkreisbogen
 (setq bt (sqrt (- (* rt rt) (* rg rg))))
 (setq gamma (- (/ bt rg) (atan (/ bt rg))))
 (setq bk (sqrt (- (* rk rk) (* rg rg))))
 (setq delta (- (/ bk rg) (atan (/ bk rg))))        ;Startwinkel Kopfkreisbogen
 (setq winsp (+ (/ (/ p rt) 4.0) gamma))               ;Winkel der Spiegelachse
 (setq winko (+ delta (* (- winsp delta) 2.0)))       ;Endwinkel Kopfkreisbogen
 (command "_arc" "_c" "_none" mp "_none" (polar mp delta rk) "_none" (polar mp winko rk))
 (setq kobog (entlast))                                 ;sichern Kopfkreisbogen
;-------------------------------------------------------------------------------
 (setq winfu (- gamma (/ p rt 4.0))) 
 (setq v (/ zfr (+ zfr rf)))
 (setq winzfr (- (* 2.0 pi) (/ v (sqrt (- 1 (* v v))))))
 (setq u (sqrt (- (* (+ zfr rf) (+ zfr rf)) (* zfr zfr))))
 (setq p1 (polar mp winfu rf))                             ;Start Fukreisbogen
 (setq p2 (polar mp winzfr rf))                               ;Start Furundung
 (setq p3 (polar mp 0 u))           ;Start Tangente zur Evolvente am Grundkreis
 (setq p4 (polar mp 0 rg))                      ;Start Evolvente vom Grundkreis
;------------------------------------------------------------------------------
                  ;zeichnen rechte Zahnflanke
 (setq beta 0.0)
 (setq s 0.0)
 (setq alpha 0.0)
 (command "_pline" "_none" p1 "_a" "_ce" "_none" mp "_none" p2 "_none" p3 "_l" "_none" p4)
 (while (< s rk)                           ;Schleifenanfang fr Evolventepunkte
   (setq pn                           ;Def. nchster Punkt der Evolvente Anfang
    (polar mp                            ;Polardefinition nchsten Punkt Beginn
     (if (> (setq beta (- (* pi (/                          ;wenn beta>delta...
                   (setq alpha (+ alpha (/ 15.0 genau))
                 ) 180.0)
             ) 
            (atan (/ (setq b (/ (* pi rg alpha) 180.0)) rg)))
           ) delta)
      (setq beta delta) beta                ;...dann beta=delta setzen sonst...
     )                                                  ;...mit beta fortfahren
     (if (> (setq s (sqrt (+ (* rg rg) (* b b)))) rk)             ;wenn s>rk...
       (setq s rk) s)               ;...dann s=rk setzen sonst mit s fortfahren
    )                                  ;Polardefinition fr nchsten Punkt Ende
   )                                    ;Def. nchster Punkt der Evolvente Ende
   (command pn)                                        ;zeichnen nchsten Punkt
 )                                                               ;Ende Schleife
 (command "")
 (setq rzfl (entlast))                               ;rechte Zahnflanke sichern
;-------------------------------------------------------------------------------
                             ;Zahnkranz zeichnen
 (command "_mirror" rzfl "" "_none" mp "_none" (polar mp winsp rt) "") ;spiegeln re. Zahnflanke
 (setq lzfl (entlast))                                ;linke Zahnflanke sichern
 (command "_pedit" rzfl "_j" kobog lzfl  "" "")     ;Polylinie Zahn kompl. bilden
 (setq zahn (entlast))                                            ;Sichern Zahn
 (command "_array" zahn "" "_p" mp z "" "")                   ;Reihe mit Anzahl z
 (command "_pedit" zahn "_j" (ssget "x" '((8 . "zr"))) "")  ;Polylinie Zahnrad
 ;-----------------------------------------------------------------------------
                        ;zeichnen Teilkreis und Zentrumkreuz
  (list 
   (command "_layer" "_set" L_CENTER "")
   (command "_circle" "_none" mp "_none" rt)
   (command "_line" "_none" (polar mp winsp (+ rk uel)) 
                    "_none" (polar mp (+ pi winsp) (+ rk uel)) "")
   (command "_line" "_none" (polar mp (+ (/ pi 2.0) winsp) (+ rk uel)) 
                    "_none" (polar mp (+ pi (/ pi 2.0) winsp) (+ rk uel)) "")
  )
;-------------------------------------------------------------------------------
; (setq ausg (getstring "Bestimmungsgren ausgeben J / <N> : "))
 (setq ausg "J")
 (if (or (= ausg "j") (= ausg "J"))
  (list
   (list (princ "\n\n Anzahl der Zhne     : ") (princ z))
   (list (princ "\n Modul                : ") (princ m))
   (list (princ "\n Teilung              : ") (princ p) (princ " mm"))
   (list (princ "\n Teilkreisdurchmesser : ") (princ dt) (princ " mm"))
   (list (princ "\n Kopfkreisdurchmesser : ") (princ dk) (princ " mm"))
   (list (princ "\n Fukreisdurchmesser  : ") (princ df) (princ " mm"))  
   (list (princ "\n Grundkreisdurchmesser: ") (princ dg) (princ " mm"))
   (list (princ "\n Kopfspiel            : ") (princ c) (princ " mm"))
   (list (princ "\n Eingriffwinkel       : ") (princ egwin) (princ ""))
   (list (princ "\n Zahnfurundung       : ") (princ zfr) (princ " mm"))
  )
 )
 (setvar "cmdecho" scmde)                       ;Rcksetzen der Systemvariablen
 (setvar "blipmode" sblip)
 (setvar "clayer" oldlayer)
 (princ)
)